{% extends "base.html" %}

{% block title %}{{ application_name }} Development Console - Datastore Admin {% endblock %}

{% block head %}
{% endblock %}

{% block breadcrumbs %}
  <span class="item"><a href="">Datastore Admin</a></span>
{% endblock %}

{% block body %}
  <iframe style="border:0px solid #ffffff"
      frameborder="0"
      src="/_ah/datastore_admin"
      width="100%"
      height="1024px">
    <p>Your browser does not support iframes.</p>
  </iframe>
{% endblock %}

{% block final %}
{% endblock %}
