{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}
{% block submit_buttons_bottom %}
  {% if original.status == "done" %}
    {{block.super}}
  {% else %}
    {% with show_delete_link=False %}
      {{block.super}}
    {% endwith %}
  {% endif %}
{% endblock %}
