{% load admin_modify %}
<div class="{{ class_names }}" >
{% for bound_field in bound_fields %}{{ bound_field.html_error_list }}{% endfor %}
{% for bound_field in bound_fields %}
  {% if bound_field.has_label_first %}{% field_label bound_field %}{% endif %}
  {% field_widget bound_field %}
  {% if not bound_field.has_label_first %}{% field_label bound_field %}{% endif %}
  {% if bound_field.field.help_text %}<p class="help">{{ bound_field.field.help_text }}</p>{% endif %}
{% endfor %}
</div>
