<!doctype html>
<html>
  <head>
    <title> Appstats Analytics </title>
  </head>
  <body>
    <center>
      <p> <b> Analyzing Appstats Snapshot from {{source}} </b> </p>
    </center>

    <p> <b> Sorry!! {{errormessage}} </b> </p>

    <p> <a href = ?> Analyze another Appstats snapshot </a> </p>
  </body>
</html>
